#!/usr/bin/perl -w
#*******************************************************************************
# Copyright (c) 2005, 2010 IBM Corporation and others.
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#
# Contributors:
# IBM Corporation - initial API and implementation
# David McKnight   (IBM)   - [254785] [dstore] RSE Server assumes home directory on target machine
# David McKnight   (IBM)   - [262013] [dstore][unix] RSE Daemon fails to start server on HP-UX
# David McKnight   (IBM)   - [270833] Unify rseserver auth.pl to not use "su -p" on any Platform
# Jeff Johnston    (Red Hat) - [364859] Support Kerberos
#*******************************************************************************

use Shell;
use Authen::PAM;
use POSIX qw(ttyname);

if (!defined($ARGV[0]) || !defined($ARGV[1]) || !defined($ARGV[2]) || !defined($ARGV[3]) || !defined($ARGV[4]))
{
    print("command usage:\n");
   print("auth.pl USER, PATH, PORT, TIMEOUT, TICKET\n");
}
else
{
  $userIN    = $ARGV[0];
  $pathIN    = $ARGV[1];
  $portIN    = $ARGV[2];
  $timeoutIN = $ARGV[3];
  $ticketIN  = $ARGV[4];

 if (defined($ARGV[5]))
  {
    $javaHome = $ARGV[5];
    $javaExe = "$javaHome/bin/java";
  }
  else
  {
    $javaExe = "java"
  }

  $pwdIN = <STDIN>;
  chomp($pwdIN);


  @passwdStruct = getpwnam($userIN);

  sub my_conv_func {
    my @res;
    while ( @_ ) {
        my $code = shift;
        my $msg = shift;
        my $ans = "";

        $ans = $userIN if ($code == PAM_PROMPT_ECHO_ON() );
        $ans = $pwdIN if ($code == PAM_PROMPT_ECHO_OFF() );

        push @res, (PAM_SUCCESS(),$ans);
    }
    push @res, PAM_SUCCESS();
    return @res;
  }

  if (@passwdStruct == 0)
  {
     print("invalid user name\n");
     0;
  }
  else
  {
    $dir=$passwdStruct[7]; # get the user's home dir
    # now authenticate the password using Authen::PAM instead
    # of using passwd in passwdStruct because it does not support
    # kerberos
    $service="login";
    $tty_name = ttyname(fileno(STDIN));
    ref($pamh = new Authen::PAM($service, $userIN, \&my_conv_func)) ||
         die "Error code $pamh during PAM init!";

    $res = $pamh->pam_set_item(PAM_TTY(), $tty_name);
    $res = $pamh->pam_authenticate;
    $classpath=$ENV{CLASSPATH};
    $suOptions="-";

    if ($res == PAM_SUCCESS())
    {
		print("success\n");

		# check for the existence of a home directory
		$homeDir=$dir;
		if (!(-e $homeDir))
	    {
	      $homeDir="/tmp/" . $userIN;
	    }	 
		
		system("su $suOptions $userIN -c '$javaExe -Duser.home=$homeDir -cp $classpath -DA_PLUGIN_PATH=$pathIN -DDSTORE_SPIRIT_ON=true org.eclipse.dstore.core.server.Server $portIN $timeoutIN $ticketIN'");
		1;
    }
    else
    {
		print("incorrect password\n");
    	0;
    }
  }
}
